try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
	from zcrmsdk.src.com.zoho.crm.api.tags.record_action_handler import RecordActionHandler
except Exception:
	from ..exception import SDKException
	from ..util import Constants
	from .record_action_handler import RecordActionHandler


class RecordActionWrapper(RecordActionHandler):
	def __init__(self):
		"""Creates an instance of RecordActionWrapper"""
		super().__init__()

		self.__data = None
		self.__wf_scheduler = None
		self.__success_count = None
		self.__locked_count = None
		self.__key_modified = dict()

	def get_data(self):
		"""
		The method to get the data

		Returns:
			list: An instance of list
		"""

		return self.__data

	def set_data(self, data):
		"""
		The method to set the value to data

		Parameters:
			data (list) : An instance of list
		"""

		if data is not None and not isinstance(data, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: data EXPECTED TYPE: list', None, None)
		
		self.__data = data
		self.__key_modified['data'] = 1

	def get_wf_scheduler(self):
		"""
		The method to get the wf_scheduler

		Returns:
			bool: A bool representing the wf_scheduler
		"""

		return self.__wf_scheduler

	def set_wf_scheduler(self, wf_scheduler):
		"""
		The method to set the value to wf_scheduler

		Parameters:
			wf_scheduler (bool) : A bool representing the wf_scheduler
		"""

		if wf_scheduler is not None and not isinstance(wf_scheduler, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: wf_scheduler EXPECTED TYPE: bool', None, None)
		
		self.__wf_scheduler = wf_scheduler
		self.__key_modified['wf_scheduler'] = 1

	def get_success_count(self):
		"""
		The method to get the success_count

		Returns:
			string: A string representing the success_count
		"""

		return self.__success_count

	def set_success_count(self, success_count):
		"""
		The method to set the value to success_count

		Parameters:
			success_count (string) : A string representing the success_count
		"""

		if success_count is not None and not isinstance(success_count, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: success_count EXPECTED TYPE: str', None, None)
		
		self.__success_count = success_count
		self.__key_modified['success_count'] = 1

	def get_locked_count(self):
		"""
		The method to get the locked_count

		Returns:
			int: An int representing the locked_count
		"""

		return self.__locked_count

	def set_locked_count(self, locked_count):
		"""
		The method to set the value to locked_count

		Parameters:
			locked_count (int) : An int representing the locked_count
		"""

		if locked_count is not None and not isinstance(locked_count, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: locked_count EXPECTED TYPE: int', None, None)
		
		self.__locked_count = locked_count
		self.__key_modified['locked_count'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
    from zcrmsdk.src.com.zoho.crm.api.util.header_param_validator import HeaderParamValidator
    from zcrmsdk.src.com.zoho.crm.api.param import Param
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
    from zcrmsdk.src.com.zoho.crm.api.util.constants import Constants
except Exception:
    from .util import HeaderParamValidator
    from .param import Param
    from ..api.exception import SDKException
    from ..api.util import Constants


class ParameterMap(object):

    """
    This class represents the HTTP parameter name and value.
    """

    def __init__(self):
        """Creates an instance of ParameterMap Class"""

        self.request_parameters = dict()

    def add(self, param, value):

        """
        The method to add parameter name and value.

        Parameters:
            param (Param): A Param class instance.
            value (object): An object containing the parameter value.
        """

        if param is None:
            raise SDKException(Constants.PARAMETER_NONE_ERROR, Constants.PARAM_INSTANCE_NONE_ERROR)

        param_name = param.name
        
        if param_name is None:
            raise SDKException(Constants.PARAM_NAME_NONE_ERROR, Constants.PARAM_NAME_NONE_ERROR_MESSAGE)

        if value is None:
            raise SDKException(Constants.PARAMETER_NONE_ERROR, param_name + Constants.NONE_VALUE_ERROR_MESSAGE)

        class_name = param.class_name

        if class_name is not None:
            value = HeaderParamValidator().validate(param, value)

        if param_name not in self.request_parameters:
            self.request_parameters[param_name] = str(value)

        else:
            parameter_value = self.request_parameters[param_name]
            self.request_parameters[param_name] = parameter_value + ',' + str(value)



from .api_exception import APIException
from .assignment_rules_operations import AssignmentRulesOperations, GetAssignmentRuleParam
from .response_handler import ResponseHandler
from .assignment_rule import AssignmentRule
from .default_user import DefaultUser
from .response_wrapper import ResponseWrapper



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class CustomView(object):
	def __init__(self):
		"""Creates an instance of CustomView"""

		self.__id = None
		self.__name = None
		self.__system_name = None
		self.__display_value = None
		self.__access_type = None
		self.__category = None
		self.__sort_by = None
		self.__sort_order = None
		self.__favorite = None
		self.__default = None
		self.__system_defined = None
		self.__criteria = None
		self.__shared_to = None
		self.__fields = None
		self.__modified_time = None
		self.__modified_by = None
		self.__last_accessed_time = None
		self.__created_by = None
		self.__key_modified = dict()

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_system_name(self):
		"""
		The method to get the system_name

		Returns:
			string: A string representing the system_name
		"""

		return self.__system_name

	def set_system_name(self, system_name):
		"""
		The method to set the value to system_name

		Parameters:
			system_name (string) : A string representing the system_name
		"""

		if system_name is not None and not isinstance(system_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: system_name EXPECTED TYPE: str', None, None)
		
		self.__system_name = system_name
		self.__key_modified['system_name'] = 1

	def get_display_value(self):
		"""
		The method to get the display_value

		Returns:
			string: A string representing the display_value
		"""

		return self.__display_value

	def set_display_value(self, display_value):
		"""
		The method to set the value to display_value

		Parameters:
			display_value (string) : A string representing the display_value
		"""

		if display_value is not None and not isinstance(display_value, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: display_value EXPECTED TYPE: str', None, None)
		
		self.__display_value = display_value
		self.__key_modified['display_value'] = 1

	def get_access_type(self):
		"""
		The method to get the access_type

		Returns:
			string: A string representing the access_type
		"""

		return self.__access_type

	def set_access_type(self, access_type):
		"""
		The method to set the value to access_type

		Parameters:
			access_type (string) : A string representing the access_type
		"""

		if access_type is not None and not isinstance(access_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: access_type EXPECTED TYPE: str', None, None)
		
		self.__access_type = access_type
		self.__key_modified['access_type'] = 1

	def get_category(self):
		"""
		The method to get the category

		Returns:
			string: A string representing the category
		"""

		return self.__category

	def set_category(self, category):
		"""
		The method to set the value to category

		Parameters:
			category (string) : A string representing the category
		"""

		if category is not None and not isinstance(category, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: category EXPECTED TYPE: str', None, None)
		
		self.__category = category
		self.__key_modified['category'] = 1

	def get_sort_by(self):
		"""
		The method to get the sort_by

		Returns:
			string: A string representing the sort_by
		"""

		return self.__sort_by

	def set_sort_by(self, sort_by):
		"""
		The method to set the value to sort_by

		Parameters:
			sort_by (string) : A string representing the sort_by
		"""

		if sort_by is not None and not isinstance(sort_by, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: sort_by EXPECTED TYPE: str', None, None)
		
		self.__sort_by = sort_by
		self.__key_modified['sort_by'] = 1

	def get_sort_order(self):
		"""
		The method to get the sort_order

		Returns:
			string: A string representing the sort_order
		"""

		return self.__sort_order

	def set_sort_order(self, sort_order):
		"""
		The method to set the value to sort_order

		Parameters:
			sort_order (string) : A string representing the sort_order
		"""

		if sort_order is not None and not isinstance(sort_order, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: sort_order EXPECTED TYPE: str', None, None)
		
		self.__sort_order = sort_order
		self.__key_modified['sort_order'] = 1

	def get_favorite(self):
		"""
		The method to get the favorite

		Returns:
			int: An int representing the favorite
		"""

		return self.__favorite

	def set_favorite(self, favorite):
		"""
		The method to set the value to favorite

		Parameters:
			favorite (int) : An int representing the favorite
		"""

		if favorite is not None and not isinstance(favorite, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: favorite EXPECTED TYPE: int', None, None)
		
		self.__favorite = favorite
		self.__key_modified['favorite'] = 1

	def get_default(self):
		"""
		The method to get the default

		Returns:
			bool: A bool representing the default
		"""

		return self.__default

	def set_default(self, default):
		"""
		The method to set the value to default

		Parameters:
			default (bool) : A bool representing the default
		"""

		if default is not None and not isinstance(default, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: default EXPECTED TYPE: bool', None, None)
		
		self.__default = default
		self.__key_modified['default'] = 1

	def get_system_defined(self):
		"""
		The method to get the system_defined

		Returns:
			bool: A bool representing the system_defined
		"""

		return self.__system_defined

	def set_system_defined(self, system_defined):
		"""
		The method to set the value to system_defined

		Parameters:
			system_defined (bool) : A bool representing the system_defined
		"""

		if system_defined is not None and not isinstance(system_defined, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: system_defined EXPECTED TYPE: bool', None, None)
		
		self.__system_defined = system_defined
		self.__key_modified['system_defined'] = 1

	def get_criteria(self):
		"""
		The method to get the criteria

		Returns:
			Criteria: An instance of Criteria
		"""

		return self.__criteria

	def set_criteria(self, criteria):
		"""
		The method to set the value to criteria

		Parameters:
			criteria (Criteria) : An instance of Criteria
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.custom_views.criteria import Criteria
		except Exception:
			from .criteria import Criteria

		if criteria is not None and not isinstance(criteria, Criteria):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: criteria EXPECTED TYPE: Criteria', None, None)
		
		self.__criteria = criteria
		self.__key_modified['criteria'] = 1

	def get_shared_to(self):
		"""
		The method to get the shared_to

		Returns:
			list: An instance of list
		"""

		return self.__shared_to

	def set_shared_to(self, shared_to):
		"""
		The method to set the value to shared_to

		Parameters:
			shared_to (list) : An instance of list
		"""

		if shared_to is not None and not isinstance(shared_to, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: shared_to EXPECTED TYPE: list', None, None)
		
		self.__shared_to = shared_to
		self.__key_modified['shared_to'] = 1

	def get_fields(self):
		"""
		The method to get the fields

		Returns:
			list: An instance of list
		"""

		return self.__fields

	def set_fields(self, fields):
		"""
		The method to set the value to fields

		Parameters:
			fields (list) : An instance of list
		"""

		if fields is not None and not isinstance(fields, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: fields EXPECTED TYPE: list', None, None)
		
		self.__fields = fields
		self.__key_modified['fields'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['modified_time'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['modified_by'] = 1

	def get_last_accessed_time(self):
		"""
		The method to get the last_accessed_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__last_accessed_time

	def set_last_accessed_time(self, last_accessed_time):
		"""
		The method to set the value to last_accessed_time

		Parameters:
			last_accessed_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if last_accessed_time is not None and not isinstance(last_accessed_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: last_accessed_time EXPECTED TYPE: datetime', None, None)
		
		self.__last_accessed_time = last_accessed_time
		self.__key_modified['last_accessed_time'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['created_by'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.parameter_map import ParameterMap
	from zcrmsdk.src.com.zoho.crm.api.util import APIResponse, CommonAPIHandler, Constants
	from zcrmsdk.src.com.zoho.crm.api.param import Param
except Exception:
	from ..exception import SDKException
	from ..parameter_map import ParameterMap
	from ..util import APIResponse, CommonAPIHandler, Constants
	from ..param import Param


class CustomViewsOperations(object):
	def __init__(self, module=None):
		"""
		Creates an instance of CustomViewsOperations with the given parameters

		Parameters:
			module (string) : A string representing the module
		"""

		if module is not None and not isinstance(module, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module EXPECTED TYPE: str', None, None)
		
		self.__module = module


	def get_custom_views(self, param_instance=None):
		"""
		The method to get custom views

		Parameters:
			param_instance (ParameterMap) : An instance of ParameterMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if param_instance is not None and not isinstance(param_instance, ParameterMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: param_instance EXPECTED TYPE: ParameterMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/settings/custom_views'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.add_param(Param('module', 'com.zoho.crm.api.CustomViews.GetCustomViewsParam'), self.__module)
		handler_instance.set_param(param_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.custom_views.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')

	def get_custom_view(self, id):
		"""
		The method to get custom view

		Parameters:
			id (int) : An int representing the id

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/settings/custom_views/'
		api_path = api_path + str(id)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.add_param(Param('module', 'com.zoho.crm.api.CustomViews.GetCustomViewParam'), self.__module)
		try:
			from zcrmsdk.src.com.zoho.crm.api.custom_views.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')


class GetCustomViewsParam(object):
	page = Param('page', 'com.zoho.crm.api.CustomViews.GetCustomViewsParam')
	per_page = Param('per_page', 'com.zoho.crm.api.CustomViews.GetCustomViewsParam')


class GetCustomViewParam(object):
	pass



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class PickListValue(object):
	def __init__(self):
		"""Creates an instance of PickListValue"""

		self.__display_value = None
		self.__sequence_number = None
		self.__expected_data_type = None
		self.__maps = None
		self.__actual_value = None
		self.__sys_ref_name = None
		self.__type = None
		self.__id = None
		self.__key_modified = dict()

	def get_display_value(self):
		"""
		The method to get the display_value

		Returns:
			string: A string representing the display_value
		"""

		return self.__display_value

	def set_display_value(self, display_value):
		"""
		The method to set the value to display_value

		Parameters:
			display_value (string) : A string representing the display_value
		"""

		if display_value is not None and not isinstance(display_value, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: display_value EXPECTED TYPE: str', None, None)
		
		self.__display_value = display_value
		self.__key_modified['display_value'] = 1

	def get_sequence_number(self):
		"""
		The method to get the sequence_number

		Returns:
			int: An int representing the sequence_number
		"""

		return self.__sequence_number

	def set_sequence_number(self, sequence_number):
		"""
		The method to set the value to sequence_number

		Parameters:
			sequence_number (int) : An int representing the sequence_number
		"""

		if sequence_number is not None and not isinstance(sequence_number, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: sequence_number EXPECTED TYPE: int', None, None)
		
		self.__sequence_number = sequence_number
		self.__key_modified['sequence_number'] = 1

	def get_expected_data_type(self):
		"""
		The method to get the expected_data_type

		Returns:
			string: A string representing the expected_data_type
		"""

		return self.__expected_data_type

	def set_expected_data_type(self, expected_data_type):
		"""
		The method to set the value to expected_data_type

		Parameters:
			expected_data_type (string) : A string representing the expected_data_type
		"""

		if expected_data_type is not None and not isinstance(expected_data_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: expected_data_type EXPECTED TYPE: str', None, None)
		
		self.__expected_data_type = expected_data_type
		self.__key_modified['expected_data_type'] = 1

	def get_maps(self):
		"""
		The method to get the maps

		Returns:
			list: An instance of list
		"""

		return self.__maps

	def set_maps(self, maps):
		"""
		The method to set the value to maps

		Parameters:
			maps (list) : An instance of list
		"""

		if maps is not None and not isinstance(maps, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: maps EXPECTED TYPE: list', None, None)
		
		self.__maps = maps
		self.__key_modified['maps'] = 1

	def get_actual_value(self):
		"""
		The method to get the actual_value

		Returns:
			string: A string representing the actual_value
		"""

		return self.__actual_value

	def set_actual_value(self, actual_value):
		"""
		The method to set the value to actual_value

		Parameters:
			actual_value (string) : A string representing the actual_value
		"""

		if actual_value is not None and not isinstance(actual_value, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: actual_value EXPECTED TYPE: str', None, None)
		
		self.__actual_value = actual_value
		self.__key_modified['actual_value'] = 1

	def get_sys_ref_name(self):
		"""
		The method to get the sys_ref_name

		Returns:
			string: A string representing the sys_ref_name
		"""

		return self.__sys_ref_name

	def set_sys_ref_name(self, sys_ref_name):
		"""
		The method to set the value to sys_ref_name

		Parameters:
			sys_ref_name (string) : A string representing the sys_ref_name
		"""

		if sys_ref_name is not None and not isinstance(sys_ref_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: sys_ref_name EXPECTED TYPE: str', None, None)
		
		self.__sys_ref_name = sys_ref_name
		self.__key_modified['sys_ref_name'] = 1

	def get_type(self):
		"""
		The method to get the type

		Returns:
			string: A string representing the type
		"""

		return self.__type

	def set_type(self, type):
		"""
		The method to set the value to type

		Parameters:
			type (string) : A string representing the type
		"""

		if type is not None and not isinstance(type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: type EXPECTED TYPE: str', None, None)
		
		self.__type = type
		self.__key_modified['type'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Options(object):
	def __init__(self):
		"""Creates an instance of Options"""
		pass




from .private import Private
from .related_details import RelatedDetails
from .multi_user_lookup import MultiUserLookup
from .crypt import Crypt
from .module import Module
from .view_type import ViewType
from .lookup_field import LookupField
from .fields_operations import FieldsOperations, GetFieldsParam, GetFieldParam
from .multi_select_lookup import MultiSelectLookup
from .tool_tip import ToolTip
from .api_exception import APIException
from .external import External
from .response_handler import ResponseHandler
from .field import Field
from .association_details import AssociationDetails
from .unique import Unique
from .history_tracking import HistoryTracking
from .response_wrapper import ResponseWrapper
from .formula import Formula
from .currency import Currency
from .pick_list_value import PickListValue
from .auto_number import AutoNumber



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.parameter_map import ParameterMap
	from zcrmsdk.src.com.zoho.crm.api.util import APIResponse, CommonAPIHandler, Constants
	from zcrmsdk.src.com.zoho.crm.api.param import Param
except Exception:
	from ..exception import SDKException
	from ..parameter_map import ParameterMap
	from ..util import APIResponse, CommonAPIHandler, Constants
	from ..param import Param


class NotificationOperations(object):
	def __init__(self):
		"""Creates an instance of NotificationOperations"""
		pass

	def enable_notifications(self, request):
		"""
		The method to enable notifications

		Parameters:
			request (BodyWrapper) : An instance of BodyWrapper

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.body_wrapper import BodyWrapper
		except Exception:
			from .body_wrapper import BodyWrapper

		if request is not None and not isinstance(request, BodyWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: BodyWrapper', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/actions/watch'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_POST)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_CREATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_mandatory_checker(True)
		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def get_notification_details(self, param_instance=None):
		"""
		The method to get notification details

		Parameters:
			param_instance (ParameterMap) : An instance of ParameterMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if param_instance is not None and not isinstance(param_instance, ParameterMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: param_instance EXPECTED TYPE: ParameterMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/actions/watch'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.set_param(param_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')

	def update_notifications(self, request):
		"""
		The method to update notifications

		Parameters:
			request (BodyWrapper) : An instance of BodyWrapper

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.body_wrapper import BodyWrapper
		except Exception:
			from .body_wrapper import BodyWrapper

		if request is not None and not isinstance(request, BodyWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: BodyWrapper', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/actions/watch'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_PUT)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_UPDATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_mandatory_checker(True)
		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def update_notification(self, request):
		"""
		The method to update notification

		Parameters:
			request (BodyWrapper) : An instance of BodyWrapper

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.body_wrapper import BodyWrapper
		except Exception:
			from .body_wrapper import BodyWrapper

		if request is not None and not isinstance(request, BodyWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: BodyWrapper', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/actions/watch'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_PATCH)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_UPDATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_mandatory_checker(True)
		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def disable_notifications(self, param_instance=None):
		"""
		The method to disable notifications

		Parameters:
			param_instance (ParameterMap) : An instance of ParameterMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if param_instance is not None and not isinstance(param_instance, ParameterMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: param_instance EXPECTED TYPE: ParameterMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/actions/watch'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_DELETE)
		handler_instance.set_category_method(Constants.REQUEST_METHOD_DELETE)
		handler_instance.set_param(param_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def disable_notification(self, request):
		"""
		The method to disable notification

		Parameters:
			request (BodyWrapper) : An instance of BodyWrapper

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.body_wrapper import BodyWrapper
		except Exception:
			from .body_wrapper import BodyWrapper

		if request is not None and not isinstance(request, BodyWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: BodyWrapper', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/actions/watch'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_PATCH)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_UPDATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_mandatory_checker(True)
		try:
			from zcrmsdk.src.com.zoho.crm.api.notification.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')


class GetNotificationDetailsParam(object):
	page = Param('page', 'com.zoho.crm.api.Notification.GetNotificationDetailsParam')
	per_page = Param('per_page', 'com.zoho.crm.api.Notification.GetNotificationDetailsParam')
	channel_id = Param('channel_id', 'com.zoho.crm.api.Notification.GetNotificationDetailsParam')
	module = Param('module', 'com.zoho.crm.api.Notification.GetNotificationDetailsParam')


class DisableNotificationsParam(object):
	channel_ids = Param('channel_ids', 'com.zoho.crm.api.Notification.DisableNotificationsParam')



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Info(object):
	def __init__(self):
		"""Creates an instance of Info"""

		self.__per_page = None
		self.__count = None
		self.__page = None
		self.__more_records = None
		self.__key_modified = dict()

	def get_per_page(self):
		"""
		The method to get the per_page

		Returns:
			int: An int representing the per_page
		"""

		return self.__per_page

	def set_per_page(self, per_page):
		"""
		The method to set the value to per_page

		Parameters:
			per_page (int) : An int representing the per_page
		"""

		if per_page is not None and not isinstance(per_page, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: per_page EXPECTED TYPE: int', None, None)
		
		self.__per_page = per_page
		self.__key_modified['per_page'] = 1

	def get_count(self):
		"""
		The method to get the count

		Returns:
			int: An int representing the count
		"""

		return self.__count

	def set_count(self, count):
		"""
		The method to set the value to count

		Parameters:
			count (int) : An int representing the count
		"""

		if count is not None and not isinstance(count, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: count EXPECTED TYPE: int', None, None)
		
		self.__count = count
		self.__key_modified['count'] = 1

	def get_page(self):
		"""
		The method to get the page

		Returns:
			int: An int representing the page
		"""

		return self.__page

	def set_page(self, page):
		"""
		The method to set the value to page

		Parameters:
			page (int) : An int representing the page
		"""

		if page is not None and not isinstance(page, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: page EXPECTED TYPE: int', None, None)
		
		self.__page = page
		self.__key_modified['page'] = 1

	def get_more_records(self):
		"""
		The method to get the more_records

		Returns:
			bool: A bool representing the more_records
		"""

		return self.__more_records

	def set_more_records(self, more_records):
		"""
		The method to set the value to more_records

		Parameters:
			more_records (bool) : A bool representing the more_records
		"""

		if more_records is not None and not isinstance(more_records, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: more_records EXPECTED TYPE: bool', None, None)
		
		self.__more_records = more_records
		self.__key_modified['more_records'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class ShareRecord(object):
	def __init__(self):
		"""Creates an instance of ShareRecord"""

		self.__share_related_records = None
		self.__shared_through = None
		self.__shared_time = None
		self.__permission = None
		self.__shared_by = None
		self.__user = None
		self.__key_modified = dict()

	def get_share_related_records(self):
		"""
		The method to get the share_related_records

		Returns:
			bool: A bool representing the share_related_records
		"""

		return self.__share_related_records

	def set_share_related_records(self, share_related_records):
		"""
		The method to set the value to share_related_records

		Parameters:
			share_related_records (bool) : A bool representing the share_related_records
		"""

		if share_related_records is not None and not isinstance(share_related_records, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: share_related_records EXPECTED TYPE: bool', None, None)
		
		self.__share_related_records = share_related_records
		self.__key_modified['share_related_records'] = 1

	def get_shared_through(self):
		"""
		The method to get the shared_through

		Returns:
			SharedThrough: An instance of SharedThrough
		"""

		return self.__shared_through

	def set_shared_through(self, shared_through):
		"""
		The method to set the value to shared_through

		Parameters:
			shared_through (SharedThrough) : An instance of SharedThrough
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.share_records.shared_through import SharedThrough
		except Exception:
			from .shared_through import SharedThrough

		if shared_through is not None and not isinstance(shared_through, SharedThrough):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: shared_through EXPECTED TYPE: SharedThrough', None, None)
		
		self.__shared_through = shared_through
		self.__key_modified['shared_through'] = 1

	def get_shared_time(self):
		"""
		The method to get the shared_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__shared_time

	def set_shared_time(self, shared_time):
		"""
		The method to set the value to shared_time

		Parameters:
			shared_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if shared_time is not None and not isinstance(shared_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: shared_time EXPECTED TYPE: datetime', None, None)
		
		self.__shared_time = shared_time
		self.__key_modified['shared_time'] = 1

	def get_permission(self):
		"""
		The method to get the permission

		Returns:
			string: A string representing the permission
		"""

		return self.__permission

	def set_permission(self, permission):
		"""
		The method to set the value to permission

		Parameters:
			permission (string) : A string representing the permission
		"""

		if permission is not None and not isinstance(permission, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: permission EXPECTED TYPE: str', None, None)
		
		self.__permission = permission
		self.__key_modified['permission'] = 1

	def get_shared_by(self):
		"""
		The method to get the shared_by

		Returns:
			User: An instance of User
		"""

		return self.__shared_by

	def set_shared_by(self, shared_by):
		"""
		The method to set the value to shared_by

		Parameters:
			shared_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if shared_by is not None and not isinstance(shared_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: shared_by EXPECTED TYPE: User', None, None)
		
		self.__shared_by = shared_by
		self.__key_modified['shared_by'] = 1

	def get_user(self):
		"""
		The method to get the user

		Returns:
			User: An instance of User
		"""

		return self.__user

	def set_user(self, user):
		"""
		The method to set the value to user

		Parameters:
			user (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if user is not None and not isinstance(user, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: user EXPECTED TYPE: User', None, None)
		
		self.__user = user
		self.__key_modified['user'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Mail(object):
	def __init__(self):
		"""Creates an instance of Mail"""

		self.__from_1 = None
		self.__to = None
		self.__cc = None
		self.__bcc = None
		self.__reply_to = None
		self.__template = None
		self.__email = None
		self.__id = None
		self.__in_reply_to = None
		self.__scheduled_time = None
		self.__subject = None
		self.__content = None
		self.__paper_type = None
		self.__view_type = None
		self.__mail_format = None
		self.__consent_email = None
		self.__org_email = None
		self.__attachments = None
		self.__inventory_details = None
		self.__data_subject_request = None
		self.__key_modified = dict()

	def get_from(self):
		"""
		The method to get the from

		Returns:
			UserAddress: An instance of UserAddress
		"""

		return self.__from_1

	def set_from(self, from_1):
		"""
		The method to set the value to from

		Parameters:
			from_1 (UserAddress) : An instance of UserAddress
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.send_mail.user_address import UserAddress
		except Exception:
			from .user_address import UserAddress

		if from_1 is not None and not isinstance(from_1, UserAddress):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: from_1 EXPECTED TYPE: UserAddress', None, None)
		
		self.__from_1 = from_1
		self.__key_modified['from'] = 1

	def get_to(self):
		"""
		The method to get the to

		Returns:
			list: An instance of list
		"""

		return self.__to

	def set_to(self, to):
		"""
		The method to set the value to to

		Parameters:
			to (list) : An instance of list
		"""

		if to is not None and not isinstance(to, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: to EXPECTED TYPE: list', None, None)
		
		self.__to = to
		self.__key_modified['to'] = 1

	def get_cc(self):
		"""
		The method to get the cc

		Returns:
			list: An instance of list
		"""

		return self.__cc

	def set_cc(self, cc):
		"""
		The method to set the value to cc

		Parameters:
			cc (list) : An instance of list
		"""

		if cc is not None and not isinstance(cc, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: cc EXPECTED TYPE: list', None, None)
		
		self.__cc = cc
		self.__key_modified['cc'] = 1

	def get_bcc(self):
		"""
		The method to get the bcc

		Returns:
			list: An instance of list
		"""

		return self.__bcc

	def set_bcc(self, bcc):
		"""
		The method to set the value to bcc

		Parameters:
			bcc (list) : An instance of list
		"""

		if bcc is not None and not isinstance(bcc, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: bcc EXPECTED TYPE: list', None, None)
		
		self.__bcc = bcc
		self.__key_modified['bcc'] = 1

	def get_reply_to(self):
		"""
		The method to get the reply_to

		Returns:
			UserAddress: An instance of UserAddress
		"""

		return self.__reply_to

	def set_reply_to(self, reply_to):
		"""
		The method to set the value to reply_to

		Parameters:
			reply_to (UserAddress) : An instance of UserAddress
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.send_mail.user_address import UserAddress
		except Exception:
			from .user_address import UserAddress

		if reply_to is not None and not isinstance(reply_to, UserAddress):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: reply_to EXPECTED TYPE: UserAddress', None, None)
		
		self.__reply_to = reply_to
		self.__key_modified['reply_to'] = 1

	def get_template(self):
		"""
		The method to get the template

		Returns:
			Template: An instance of Template
		"""

		return self.__template

	def set_template(self, template):
		"""
		The method to set the value to template

		Parameters:
			template (Template) : An instance of Template
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.send_mail.template import Template
		except Exception:
			from .template import Template

		if template is not None and not isinstance(template, Template):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: template EXPECTED TYPE: Template', None, None)
		
		self.__template = template
		self.__key_modified['template'] = 1

	def get_email(self):
		"""
		The method to get the email

		Returns:
			int: An int representing the email
		"""

		return self.__email

	def set_email(self, email):
		"""
		The method to set the value to email

		Parameters:
			email (int) : An int representing the email
		"""

		if email is not None and not isinstance(email, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: email EXPECTED TYPE: int', None, None)
		
		self.__email = email
		self.__key_modified['email'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_in_reply_to(self):
		"""
		The method to get the in_reply_to

		Returns:
			string: A string representing the in_reply_to
		"""

		return self.__in_reply_to

	def set_in_reply_to(self, in_reply_to):
		"""
		The method to set the value to in_reply_to

		Parameters:
			in_reply_to (string) : A string representing the in_reply_to
		"""

		if in_reply_to is not None and not isinstance(in_reply_to, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: in_reply_to EXPECTED TYPE: str', None, None)
		
		self.__in_reply_to = in_reply_to
		self.__key_modified['in_reply_to'] = 1

	def get_scheduled_time(self):
		"""
		The method to get the scheduled_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__scheduled_time

	def set_scheduled_time(self, scheduled_time):
		"""
		The method to set the value to scheduled_time

		Parameters:
			scheduled_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if scheduled_time is not None and not isinstance(scheduled_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: scheduled_time EXPECTED TYPE: datetime', None, None)
		
		self.__scheduled_time = scheduled_time
		self.__key_modified['scheduled_time'] = 1

	def get_subject(self):
		"""
		The method to get the subject

		Returns:
			string: A string representing the subject
		"""

		return self.__subject

	def set_subject(self, subject):
		"""
		The method to set the value to subject

		Parameters:
			subject (string) : A string representing the subject
		"""

		if subject is not None and not isinstance(subject, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: subject EXPECTED TYPE: str', None, None)
		
		self.__subject = subject
		self.__key_modified['subject'] = 1

	def get_content(self):
		"""
		The method to get the content

		Returns:
			string: A string representing the content
		"""

		return self.__content

	def set_content(self, content):
		"""
		The method to set the value to content

		Parameters:
			content (string) : A string representing the content
		"""

		if content is not None and not isinstance(content, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: content EXPECTED TYPE: str', None, None)
		
		self.__content = content
		self.__key_modified['content'] = 1

	def get_paper_type(self):
		"""
		The method to get the paper_type

		Returns:
			string: A string representing the paper_type
		"""

		return self.__paper_type

	def set_paper_type(self, paper_type):
		"""
		The method to set the value to paper_type

		Parameters:
			paper_type (string) : A string representing the paper_type
		"""

		if paper_type is not None and not isinstance(paper_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: paper_type EXPECTED TYPE: str', None, None)
		
		self.__paper_type = paper_type
		self.__key_modified['paper_type'] = 1

	def get_view_type(self):
		"""
		The method to get the view_type

		Returns:
			string: A string representing the view_type
		"""

		return self.__view_type

	def set_view_type(self, view_type):
		"""
		The method to set the value to view_type

		Parameters:
			view_type (string) : A string representing the view_type
		"""

		if view_type is not None and not isinstance(view_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: view_type EXPECTED TYPE: str', None, None)
		
		self.__view_type = view_type
		self.__key_modified['view_type'] = 1

	def get_mail_format(self):
		"""
		The method to get the mail_format

		Returns:
			string: A string representing the mail_format
		"""

		return self.__mail_format

	def set_mail_format(self, mail_format):
		"""
		The method to set the value to mail_format

		Parameters:
			mail_format (string) : A string representing the mail_format
		"""

		if mail_format is not None and not isinstance(mail_format, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: mail_format EXPECTED TYPE: str', None, None)
		
		self.__mail_format = mail_format
		self.__key_modified['mail_format'] = 1

	def get_consent_email(self):
		"""
		The method to get the consent_email

		Returns:
			bool: A bool representing the consent_email
		"""

		return self.__consent_email

	def set_consent_email(self, consent_email):
		"""
		The method to set the value to consent_email

		Parameters:
			consent_email (bool) : A bool representing the consent_email
		"""

		if consent_email is not None and not isinstance(consent_email, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: consent_email EXPECTED TYPE: bool', None, None)
		
		self.__consent_email = consent_email
		self.__key_modified['consent_email'] = 1

	def get_org_email(self):
		"""
		The method to get the org_email

		Returns:
			bool: A bool representing the org_email
		"""

		return self.__org_email

	def set_org_email(self, org_email):
		"""
		The method to set the value to org_email

		Parameters:
			org_email (bool) : A bool representing the org_email
		"""

		if org_email is not None and not isinstance(org_email, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: org_email EXPECTED TYPE: bool', None, None)
		
		self.__org_email = org_email
		self.__key_modified['org_email'] = 1

	def get_attachments(self):
		"""
		The method to get the attachments

		Returns:
			list: An instance of list
		"""

		return self.__attachments

	def set_attachments(self, attachments):
		"""
		The method to set the value to attachments

		Parameters:
			attachments (list) : An instance of list
		"""

		if attachments is not None and not isinstance(attachments, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: attachments EXPECTED TYPE: list', None, None)
		
		self.__attachments = attachments
		self.__key_modified['attachments'] = 1

	def get_inventory_details(self):
		"""
		The method to get the inventory_details

		Returns:
			InventoryDetails: An instance of InventoryDetails
		"""

		return self.__inventory_details

	def set_inventory_details(self, inventory_details):
		"""
		The method to set the value to inventory_details

		Parameters:
			inventory_details (InventoryDetails) : An instance of InventoryDetails
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.send_mail.inventory_details import InventoryDetails
		except Exception:
			from .inventory_details import InventoryDetails

		if inventory_details is not None and not isinstance(inventory_details, InventoryDetails):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: inventory_details EXPECTED TYPE: InventoryDetails', None, None)
		
		self.__inventory_details = inventory_details
		self.__key_modified['inventory_details'] = 1

	def get_data_subject_request(self):
		"""
		The method to get the data_subject_request

		Returns:
			DataSubjectRequest: An instance of DataSubjectRequest
		"""

		return self.__data_subject_request

	def set_data_subject_request(self, data_subject_request):
		"""
		The method to set the value to data_subject_request

		Parameters:
			data_subject_request (DataSubjectRequest) : An instance of DataSubjectRequest
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.send_mail.data_subject_request import DataSubjectRequest
		except Exception:
			from .data_subject_request import DataSubjectRequest

		if data_subject_request is not None and not isinstance(data_subject_request, DataSubjectRequest):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: data_subject_request EXPECTED TYPE: DataSubjectRequest', None, None)
		
		self.__data_subject_request = data_subject_request
		self.__key_modified['data_subject_request'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Territory(object):
	def __init__(self):
		"""Creates an instance of Territory"""

		self.__created_time = None
		self.__modified_time = None
		self.__manager = None
		self.__account_rule_criteria = None
		self.__deal_rule_criteria = None
		self.__name = None
		self.__modified_by = None
		self.__description = None
		self.__id = None
		self.__created_by = None
		self.__reporting_to = None
		self.__permission_type = None
		self.__key_modified = dict()

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['created_time'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['modified_time'] = 1

	def get_manager(self):
		"""
		The method to get the manager

		Returns:
			User: An instance of User
		"""

		return self.__manager

	def set_manager(self, manager):
		"""
		The method to set the value to manager

		Parameters:
			manager (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if manager is not None and not isinstance(manager, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: manager EXPECTED TYPE: User', None, None)
		
		self.__manager = manager
		self.__key_modified['manager'] = 1

	def get_account_rule_criteria(self):
		"""
		The method to get the account_rule_criteria

		Returns:
			Criteria: An instance of Criteria
		"""

		return self.__account_rule_criteria

	def set_account_rule_criteria(self, account_rule_criteria):
		"""
		The method to set the value to account_rule_criteria

		Parameters:
			account_rule_criteria (Criteria) : An instance of Criteria
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.customviews import Criteria
		except Exception:
			from ..custom_views import Criteria

		if account_rule_criteria is not None and not isinstance(account_rule_criteria, Criteria):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: account_rule_criteria EXPECTED TYPE: Criteria', None, None)
		
		self.__account_rule_criteria = account_rule_criteria
		self.__key_modified['account_rule_criteria'] = 1

	def get_deal_rule_criteria(self):
		"""
		The method to get the deal_rule_criteria

		Returns:
			Criteria: An instance of Criteria
		"""

		return self.__deal_rule_criteria

	def set_deal_rule_criteria(self, deal_rule_criteria):
		"""
		The method to set the value to deal_rule_criteria

		Parameters:
			deal_rule_criteria (Criteria) : An instance of Criteria
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.customviews import Criteria
		except Exception:
			from ..custom_views import Criteria

		if deal_rule_criteria is not None and not isinstance(deal_rule_criteria, Criteria):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: deal_rule_criteria EXPECTED TYPE: Criteria', None, None)
		
		self.__deal_rule_criteria = deal_rule_criteria
		self.__key_modified['deal_rule_criteria'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['modified_by'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['created_by'] = 1

	def get_reporting_to(self):
		"""
		The method to get the reporting_to

		Returns:
			Territory: An instance of Territory
		"""

		return self.__reporting_to

	def set_reporting_to(self, reporting_to):
		"""
		The method to set the value to reporting_to

		Parameters:
			reporting_to (Territory) : An instance of Territory
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.territories.territory import Territory
		except Exception:
			from .territory import Territory

		if reporting_to is not None and not isinstance(reporting_to, Territory):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: reporting_to EXPECTED TYPE: Territory', None, None)
		
		self.__reporting_to = reporting_to
		self.__key_modified['reporting_to'] = 1

	def get_permission_type(self):
		"""
		The method to get the permission_type

		Returns:
			string: A string representing the permission_type
		"""

		return self.__permission_type

	def set_permission_type(self, permission_type):
		"""
		The method to set the value to permission_type

		Parameters:
			permission_type (string) : A string representing the permission_type
		"""

		if permission_type is not None and not isinstance(permission_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: permission_type EXPECTED TYPE: str', None, None)
		
		self.__permission_type = permission_type
		self.__key_modified['permission_type'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class MassUpdateTerritory(object):
	def __init__(self):
		"""Creates an instance of MassUpdateTerritory"""

		self.__id = None
		self.__include_child = None
		self.__key_modified = dict()

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_include_child(self):
		"""
		The method to get the include_child

		Returns:
			bool: A bool representing the include_child
		"""

		return self.__include_child

	def set_include_child(self, include_child):
		"""
		The method to set the value to include_child

		Parameters:
			include_child (bool) : A bool representing the include_child
		"""

		if include_child is not None and not isinstance(include_child, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: include_child EXPECTED TYPE: bool', None, None)
		
		self.__include_child = include_child
		self.__key_modified['include_child'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class FieldMapping(object):
	def __init__(self):
		"""Creates an instance of FieldMapping"""

		self.__api_name = None
		self.__index = None
		self.__format = None
		self.__find_by = None
		self.__default_value = None
		self.__module = None
		self.__key_modified = dict()

	def get_api_name(self):
		"""
		The method to get the api_name

		Returns:
			string: A string representing the api_name
		"""

		return self.__api_name

	def set_api_name(self, api_name):
		"""
		The method to set the value to api_name

		Parameters:
			api_name (string) : A string representing the api_name
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__api_name = api_name
		self.__key_modified['api_name'] = 1

	def get_index(self):
		"""
		The method to get the index

		Returns:
			int: An int representing the index
		"""

		return self.__index

	def set_index(self, index):
		"""
		The method to set the value to index

		Parameters:
			index (int) : An int representing the index
		"""

		if index is not None and not isinstance(index, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: index EXPECTED TYPE: int', None, None)
		
		self.__index = index
		self.__key_modified['index'] = 1

	def get_format(self):
		"""
		The method to get the format

		Returns:
			string: A string representing the format
		"""

		return self.__format

	def set_format(self, format):
		"""
		The method to set the value to format

		Parameters:
			format (string) : A string representing the format
		"""

		if format is not None and not isinstance(format, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: format EXPECTED TYPE: str', None, None)
		
		self.__format = format
		self.__key_modified['format'] = 1

	def get_find_by(self):
		"""
		The method to get the find_by

		Returns:
			string: A string representing the find_by
		"""

		return self.__find_by

	def set_find_by(self, find_by):
		"""
		The method to set the value to find_by

		Parameters:
			find_by (string) : A string representing the find_by
		"""

		if find_by is not None and not isinstance(find_by, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: find_by EXPECTED TYPE: str', None, None)
		
		self.__find_by = find_by
		self.__key_modified['find_by'] = 1

	def get_default_value(self):
		"""
		The method to get the default_value

		Returns:
			dict: An instance of dict
		"""

		return self.__default_value

	def set_default_value(self, default_value):
		"""
		The method to set the value to default_value

		Parameters:
			default_value (dict) : An instance of dict
		"""

		if default_value is not None and not isinstance(default_value, dict):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: default_value EXPECTED TYPE: dict', None, None)
		
		self.__default_value = default_value
		self.__key_modified['default_value'] = 1

	def get_module(self):
		"""
		The method to get the module

		Returns:
			string: A string representing the module
		"""

		return self.__module

	def set_module(self, module):
		"""
		The method to set the value to module

		Parameters:
			module (string) : A string representing the module
		"""

		if module is not None and not isinstance(module, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module EXPECTED TYPE: str', None, None)
		
		self.__module = module
		self.__key_modified['module'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
    import logging
    from zcrmsdk.src.com.zoho.crm.api.exception.sdk_exception import SDKException
    from ...crm.api.util.constants import Constants
except:
    from ...crm.api.util.constants import Constants
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException


class Logger(object):

    """
    This class represents the Logger level and the file path.
    """

    def __init__(self, level, file_path=None):
        self.__level = level
        self.__file_path = file_path

    def get_level(self):
        """
        This is a getter method to get __level.

        Returns:
            string: A enum representing __level
        """

        return self.__level

    def get_file_path(self):
        """
        This is a getter method to get __file_path.

        Returns:
            string: A string representing __file_path
        """

        return self.__file_path

    @staticmethod
    def get_instance(level, file_path=None):

        """
        Creates an Logger class instance with the specified log level and file path.
        :param level: A Levels class instance containing the log level.
        :param file_path: A str containing the log file path.
        :return: A Logger class instance.
        """

        return Logger(level=level, file_path=file_path)

    import enum

    class Levels(enum.Enum):

        """
        This class represents the possible logger levels
        """

        CRITICAL = logging.CRITICAL
        ERROR = logging.ERROR
        WARNING = logging.WARNING
        INFO = logging.INFO
        DEBUG = logging.DEBUG
        NOTSET = logging.NOTSET


class SDKLogger(object):

    """
    The class to initialize the SDK logger.
    """

    def __init__(self, logger_instance):

        logger = logging.getLogger('SDKLogger')
        logger_level = logger_instance.get_level()
        logger_file_path = logger_instance.get_file_path()
        if logger_level is not None and logger_level != logging.NOTSET and logger_file_path is not None and logger_file_path != "":
            file_handler = logging.FileHandler(logger_file_path)
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(module)s - %(filename)s - %(funcName)s - %(lineno)d  - %(message)s')
            file_handler.setLevel(logger_level.name)
            file_handler.setFormatter(formatter)
            logger.addHandler(file_handler)
        if logger_level is not None and Constants.LOGGER_LEVELS.__contains__(logger_level.name):
            logger.setLevel(logger_level.name)



    @staticmethod
    def initialize(logger_instance):
        try:
            SDKLogger(logger_instance=logger_instance)
        except Exception as ex:
            raise SDKException(message=Constants.LOGGER_INITIALIZATION_ERROR, Exception=ex)



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Choice, Constants
except Exception:
	from ..exception import SDKException
	from ..util import Choice, Constants


class Module(object):
	def __init__(self):
		"""Creates an instance of Module"""

		self.__name = None
		self.__global_search_supported = None
		self.__kanban_view = None
		self.__deletable = None
		self.__description = None
		self.__creatable = None
		self.__filter_status = None
		self.__inventory_template_supported = None
		self.__modified_time = None
		self.__plural_label = None
		self.__presence_sub_menu = None
		self.__isblueprintsupported = None
		self.__triggers_supported = None
		self.__id = None
		self.__related_list_properties = None
		self.__properties = None
		self.__on_demand_properties = None
		self.__per_page = None
		self.__visibility = None
		self.__visible = None
		self.__convertable = None
		self.__editable = None
		self.__emailtemplate_support = None
		self.__profiles = None
		self.__filter_supported = None
		self.__display_field = None
		self.__search_layout_fields = None
		self.__kanban_view_supported = None
		self.__show_as_tab = None
		self.__web_link = None
		self.__sequence_number = None
		self.__singular_label = None
		self.__viewable = None
		self.__api_supported = None
		self.__api_name = None
		self.__quick_create = None
		self.__modified_by = None
		self.__generated_type = None
		self.__feeds_required = None
		self.__scoring_supported = None
		self.__webform_supported = None
		self.__arguments = None
		self.__module_name = None
		self.__business_card_field_limit = None
		self.__custom_view = None
		self.__parent_module = None
		self.__territory = None
		self.__key_modified = dict()

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_global_search_supported(self):
		"""
		The method to get the global_search_supported

		Returns:
			bool: A bool representing the global_search_supported
		"""

		return self.__global_search_supported

	def set_global_search_supported(self, global_search_supported):
		"""
		The method to set the value to global_search_supported

		Parameters:
			global_search_supported (bool) : A bool representing the global_search_supported
		"""

		if global_search_supported is not None and not isinstance(global_search_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: global_search_supported EXPECTED TYPE: bool', None, None)
		
		self.__global_search_supported = global_search_supported
		self.__key_modified['global_search_supported'] = 1

	def get_kanban_view(self):
		"""
		The method to get the kanban_view

		Returns:
			bool: A bool representing the kanban_view
		"""

		return self.__kanban_view

	def set_kanban_view(self, kanban_view):
		"""
		The method to set the value to kanban_view

		Parameters:
			kanban_view (bool) : A bool representing the kanban_view
		"""

		if kanban_view is not None and not isinstance(kanban_view, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: kanban_view EXPECTED TYPE: bool', None, None)
		
		self.__kanban_view = kanban_view
		self.__key_modified['kanban_view'] = 1

	def get_deletable(self):
		"""
		The method to get the deletable

		Returns:
			bool: A bool representing the deletable
		"""

		return self.__deletable

	def set_deletable(self, deletable):
		"""
		The method to set the value to deletable

		Parameters:
			deletable (bool) : A bool representing the deletable
		"""

		if deletable is not None and not isinstance(deletable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: deletable EXPECTED TYPE: bool', None, None)
		
		self.__deletable = deletable
		self.__key_modified['deletable'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_creatable(self):
		"""
		The method to get the creatable

		Returns:
			bool: A bool representing the creatable
		"""

		return self.__creatable

	def set_creatable(self, creatable):
		"""
		The method to set the value to creatable

		Parameters:
			creatable (bool) : A bool representing the creatable
		"""

		if creatable is not None and not isinstance(creatable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: creatable EXPECTED TYPE: bool', None, None)
		
		self.__creatable = creatable
		self.__key_modified['creatable'] = 1

	def get_filter_status(self):
		"""
		The method to get the filter_status

		Returns:
			bool: A bool representing the filter_status
		"""

		return self.__filter_status

	def set_filter_status(self, filter_status):
		"""
		The method to set the value to filter_status

		Parameters:
			filter_status (bool) : A bool representing the filter_status
		"""

		if filter_status is not None and not isinstance(filter_status, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: filter_status EXPECTED TYPE: bool', None, None)
		
		self.__filter_status = filter_status
		self.__key_modified['filter_status'] = 1

	def get_inventory_template_supported(self):
		"""
		The method to get the inventory_template_supported

		Returns:
			bool: A bool representing the inventory_template_supported
		"""

		return self.__inventory_template_supported

	def set_inventory_template_supported(self, inventory_template_supported):
		"""
		The method to set the value to inventory_template_supported

		Parameters:
			inventory_template_supported (bool) : A bool representing the inventory_template_supported
		"""

		if inventory_template_supported is not None and not isinstance(inventory_template_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: inventory_template_supported EXPECTED TYPE: bool', None, None)
		
		self.__inventory_template_supported = inventory_template_supported
		self.__key_modified['inventory_template_supported'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['modified_time'] = 1

	def get_plural_label(self):
		"""
		The method to get the plural_label

		Returns:
			string: A string representing the plural_label
		"""

		return self.__plural_label

	def set_plural_label(self, plural_label):
		"""
		The method to set the value to plural_label

		Parameters:
			plural_label (string) : A string representing the plural_label
		"""

		if plural_label is not None and not isinstance(plural_label, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: plural_label EXPECTED TYPE: str', None, None)
		
		self.__plural_label = plural_label
		self.__key_modified['plural_label'] = 1

	def get_presence_sub_menu(self):
		"""
		The method to get the presence_sub_menu

		Returns:
			bool: A bool representing the presence_sub_menu
		"""

		return self.__presence_sub_menu

	def set_presence_sub_menu(self, presence_sub_menu):
		"""
		The method to set the value to presence_sub_menu

		Parameters:
			presence_sub_menu (bool) : A bool representing the presence_sub_menu
		"""

		if presence_sub_menu is not None and not isinstance(presence_sub_menu, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: presence_sub_menu EXPECTED TYPE: bool', None, None)
		
		self.__presence_sub_menu = presence_sub_menu
		self.__key_modified['presence_sub_menu'] = 1

	def get_isblueprintsupported(self):
		"""
		The method to get the isblueprintsupported

		Returns:
			bool: A bool representing the isblueprintsupported
		"""

		return self.__isblueprintsupported

	def set_isblueprintsupported(self, isblueprintsupported):
		"""
		The method to set the value to isblueprintsupported

		Parameters:
			isblueprintsupported (bool) : A bool representing the isblueprintsupported
		"""

		if isblueprintsupported is not None and not isinstance(isblueprintsupported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: isblueprintsupported EXPECTED TYPE: bool', None, None)
		
		self.__isblueprintsupported = isblueprintsupported
		self.__key_modified['isBlueprintSupported'] = 1

	def get_triggers_supported(self):
		"""
		The method to get the triggers_supported

		Returns:
			bool: A bool representing the triggers_supported
		"""

		return self.__triggers_supported

	def set_triggers_supported(self, triggers_supported):
		"""
		The method to set the value to triggers_supported

		Parameters:
			triggers_supported (bool) : A bool representing the triggers_supported
		"""

		if triggers_supported is not None and not isinstance(triggers_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: triggers_supported EXPECTED TYPE: bool', None, None)
		
		self.__triggers_supported = triggers_supported
		self.__key_modified['triggers_supported'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_related_list_properties(self):
		"""
		The method to get the related_list_properties

		Returns:
			RelatedListProperties: An instance of RelatedListProperties
		"""

		return self.__related_list_properties

	def set_related_list_properties(self, related_list_properties):
		"""
		The method to set the value to related_list_properties

		Parameters:
			related_list_properties (RelatedListProperties) : An instance of RelatedListProperties
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.modules.related_list_properties import RelatedListProperties
		except Exception:
			from .related_list_properties import RelatedListProperties

		if related_list_properties is not None and not isinstance(related_list_properties, RelatedListProperties):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: related_list_properties EXPECTED TYPE: RelatedListProperties', None, None)
		
		self.__related_list_properties = related_list_properties
		self.__key_modified['related_list_properties'] = 1

	def get_properties(self):
		"""
		The method to get the properties

		Returns:
			list: An instance of list
		"""

		return self.__properties

	def set_properties(self, properties):
		"""
		The method to set the value to properties

		Parameters:
			properties (list) : An instance of list
		"""

		if properties is not None and not isinstance(properties, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: properties EXPECTED TYPE: list', None, None)
		
		self.__properties = properties
		self.__key_modified['$properties'] = 1

	def get_on_demand_properties(self):
		"""
		The method to get the on_demand_properties

		Returns:
			list: An instance of list
		"""

		return self.__on_demand_properties

	def set_on_demand_properties(self, on_demand_properties):
		"""
		The method to set the value to on_demand_properties

		Parameters:
			on_demand_properties (list) : An instance of list
		"""

		if on_demand_properties is not None and not isinstance(on_demand_properties, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: on_demand_properties EXPECTED TYPE: list', None, None)
		
		self.__on_demand_properties = on_demand_properties
		self.__key_modified['$on_demand_properties'] = 1

	def get_per_page(self):
		"""
		The method to get the per_page

		Returns:
			int: An int representing the per_page
		"""

		return self.__per_page

	def set_per_page(self, per_page):
		"""
		The method to set the value to per_page

		Parameters:
			per_page (int) : An int representing the per_page
		"""

		if per_page is not None and not isinstance(per_page, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: per_page EXPECTED TYPE: int', None, None)
		
		self.__per_page = per_page
		self.__key_modified['per_page'] = 1

	def get_visibility(self):
		"""
		The method to get the visibility

		Returns:
			int: An int representing the visibility
		"""

		return self.__visibility

	def set_visibility(self, visibility):
		"""
		The method to set the value to visibility

		Parameters:
			visibility (int) : An int representing the visibility
		"""

		if visibility is not None and not isinstance(visibility, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: visibility EXPECTED TYPE: int', None, None)
		
		self.__visibility = visibility
		self.__key_modified['visibility'] = 1

	def get_visible(self):
		"""
		The method to get the visible

		Returns:
			bool: A bool representing the visible
		"""

		return self.__visible

	def set_visible(self, visible):
		"""
		The method to set the value to visible

		Parameters:
			visible (bool) : A bool representing the visible
		"""

		if visible is not None and not isinstance(visible, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: visible EXPECTED TYPE: bool', None, None)
		
		self.__visible = visible
		self.__key_modified['visible'] = 1

	def get_convertable(self):
		"""
		The method to get the convertable

		Returns:
			bool: A bool representing the convertable
		"""

		return self.__convertable

	def set_convertable(self, convertable):
		"""
		The method to set the value to convertable

		Parameters:
			convertable (bool) : A bool representing the convertable
		"""

		if convertable is not None and not isinstance(convertable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: convertable EXPECTED TYPE: bool', None, None)
		
		self.__convertable = convertable
		self.__key_modified['convertable'] = 1

	def get_editable(self):
		"""
		The method to get the editable

		Returns:
			bool: A bool representing the editable
		"""

		return self.__editable

	def set_editable(self, editable):
		"""
		The method to set the value to editable

		Parameters:
			editable (bool) : A bool representing the editable
		"""

		if editable is not None and not isinstance(editable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: editable EXPECTED TYPE: bool', None, None)
		
		self.__editable = editable
		self.__key_modified['editable'] = 1

	def get_emailtemplate_support(self):
		"""
		The method to get the emailtemplate_support

		Returns:
			bool: A bool representing the emailtemplate_support
		"""

		return self.__emailtemplate_support

	def set_emailtemplate_support(self, emailtemplate_support):
		"""
		The method to set the value to emailtemplate_support

		Parameters:
			emailtemplate_support (bool) : A bool representing the emailtemplate_support
		"""

		if emailtemplate_support is not None and not isinstance(emailtemplate_support, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: emailtemplate_support EXPECTED TYPE: bool', None, None)
		
		self.__emailtemplate_support = emailtemplate_support
		self.__key_modified['emailTemplate_support'] = 1

	def get_profiles(self):
		"""
		The method to get the profiles

		Returns:
			list: An instance of list
		"""

		return self.__profiles

	def set_profiles(self, profiles):
		"""
		The method to set the value to profiles

		Parameters:
			profiles (list) : An instance of list
		"""

		if profiles is not None and not isinstance(profiles, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: profiles EXPECTED TYPE: list', None, None)
		
		self.__profiles = profiles
		self.__key_modified['profiles'] = 1

	def get_filter_supported(self):
		"""
		The method to get the filter_supported

		Returns:
			bool: A bool representing the filter_supported
		"""

		return self.__filter_supported

	def set_filter_supported(self, filter_supported):
		"""
		The method to set the value to filter_supported

		Parameters:
			filter_supported (bool) : A bool representing the filter_supported
		"""

		if filter_supported is not None and not isinstance(filter_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: filter_supported EXPECTED TYPE: bool', None, None)
		
		self.__filter_supported = filter_supported
		self.__key_modified['filter_supported'] = 1

	def get_display_field(self):
		"""
		The method to get the display_field

		Returns:
			string: A string representing the display_field
		"""

		return self.__display_field

	def set_display_field(self, display_field):
		"""
		The method to set the value to display_field

		Parameters:
			display_field (string) : A string representing the display_field
		"""

		if display_field is not None and not isinstance(display_field, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: display_field EXPECTED TYPE: str', None, None)
		
		self.__display_field = display_field
		self.__key_modified['display_field'] = 1

	def get_search_layout_fields(self):
		"""
		The method to get the search_layout_fields

		Returns:
			list: An instance of list
		"""

		return self.__search_layout_fields

	def set_search_layout_fields(self, search_layout_fields):
		"""
		The method to set the value to search_layout_fields

		Parameters:
			search_layout_fields (list) : An instance of list
		"""

		if search_layout_fields is not None and not isinstance(search_layout_fields, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: search_layout_fields EXPECTED TYPE: list', None, None)
		
		self.__search_layout_fields = search_layout_fields
		self.__key_modified['search_layout_fields'] = 1

	def get_kanban_view_supported(self):
		"""
		The method to get the kanban_view_supported

		Returns:
			bool: A bool representing the kanban_view_supported
		"""

		return self.__kanban_view_supported

	def set_kanban_view_supported(self, kanban_view_supported):
		"""
		The method to set the value to kanban_view_supported

		Parameters:
			kanban_view_supported (bool) : A bool representing the kanban_view_supported
		"""

		if kanban_view_supported is not None and not isinstance(kanban_view_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: kanban_view_supported EXPECTED TYPE: bool', None, None)
		
		self.__kanban_view_supported = kanban_view_supported
		self.__key_modified['kanban_view_supported'] = 1

	def get_show_as_tab(self):
		"""
		The method to get the show_as_tab

		Returns:
			bool: A bool representing the show_as_tab
		"""

		return self.__show_as_tab

	def set_show_as_tab(self, show_as_tab):
		"""
		The method to set the value to show_as_tab

		Parameters:
			show_as_tab (bool) : A bool representing the show_as_tab
		"""

		if show_as_tab is not None and not isinstance(show_as_tab, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: show_as_tab EXPECTED TYPE: bool', None, None)
		
		self.__show_as_tab = show_as_tab
		self.__key_modified['show_as_tab'] = 1

	def get_web_link(self):
		"""
		The method to get the web_link

		Returns:
			string: A string representing the web_link
		"""

		return self.__web_link

	def set_web_link(self, web_link):
		"""
		The method to set the value to web_link

		Parameters:
			web_link (string) : A string representing the web_link
		"""

		if web_link is not None and not isinstance(web_link, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: web_link EXPECTED TYPE: str', None, None)
		
		self.__web_link = web_link
		self.__key_modified['web_link'] = 1

	def get_sequence_number(self):
		"""
		The method to get the sequence_number

		Returns:
			int: An int representing the sequence_number
		"""

		return self.__sequence_number

	def set_sequence_number(self, sequence_number):
		"""
		The method to set the value to sequence_number

		Parameters:
			sequence_number (int) : An int representing the sequence_number
		"""

		if sequence_number is not None and not isinstance(sequence_number, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: sequence_number EXPECTED TYPE: int', None, None)
		
		self.__sequence_number = sequence_number
		self.__key_modified['sequence_number'] = 1

	def get_singular_label(self):
		"""
		The method to get the singular_label

		Returns:
			string: A string representing the singular_label
		"""

		return self.__singular_label

	def set_singular_label(self, singular_label):
		"""
		The method to set the value to singular_label

		Parameters:
			singular_label (string) : A string representing the singular_label
		"""

		if singular_label is not None and not isinstance(singular_label, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: singular_label EXPECTED TYPE: str', None, None)
		
		self.__singular_label = singular_label
		self.__key_modified['singular_label'] = 1

	def get_viewable(self):
		"""
		The method to get the viewable

		Returns:
			bool: A bool representing the viewable
		"""

		return self.__viewable

	def set_viewable(self, viewable):
		"""
		The method to set the value to viewable

		Parameters:
			viewable (bool) : A bool representing the viewable
		"""

		if viewable is not None and not isinstance(viewable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: viewable EXPECTED TYPE: bool', None, None)
		
		self.__viewable = viewable
		self.__key_modified['viewable'] = 1

	def get_api_supported(self):
		"""
		The method to get the api_supported

		Returns:
			bool: A bool representing the api_supported
		"""

		return self.__api_supported

	def set_api_supported(self, api_supported):
		"""
		The method to set the value to api_supported

		Parameters:
			api_supported (bool) : A bool representing the api_supported
		"""

		if api_supported is not None and not isinstance(api_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_supported EXPECTED TYPE: bool', None, None)
		
		self.__api_supported = api_supported
		self.__key_modified['api_supported'] = 1

	def get_api_name(self):
		"""
		The method to get the api_name

		Returns:
			string: A string representing the api_name
		"""

		return self.__api_name

	def set_api_name(self, api_name):
		"""
		The method to set the value to api_name

		Parameters:
			api_name (string) : A string representing the api_name
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__api_name = api_name
		self.__key_modified['api_name'] = 1

	def get_quick_create(self):
		"""
		The method to get the quick_create

		Returns:
			bool: A bool representing the quick_create
		"""

		return self.__quick_create

	def set_quick_create(self, quick_create):
		"""
		The method to set the value to quick_create

		Parameters:
			quick_create (bool) : A bool representing the quick_create
		"""

		if quick_create is not None and not isinstance(quick_create, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: quick_create EXPECTED TYPE: bool', None, None)
		
		self.__quick_create = quick_create
		self.__key_modified['quick_create'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['modified_by'] = 1

	def get_generated_type(self):
		"""
		The method to get the generated_type

		Returns:
			Choice: An instance of Choice
		"""

		return self.__generated_type

	def set_generated_type(self, generated_type):
		"""
		The method to set the value to generated_type

		Parameters:
			generated_type (Choice) : An instance of Choice
		"""

		if generated_type is not None and not isinstance(generated_type, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: generated_type EXPECTED TYPE: Choice', None, None)
		
		self.__generated_type = generated_type
		self.__key_modified['generated_type'] = 1

	def get_feeds_required(self):
		"""
		The method to get the feeds_required

		Returns:
			bool: A bool representing the feeds_required
		"""

		return self.__feeds_required

	def set_feeds_required(self, feeds_required):
		"""
		The method to set the value to feeds_required

		Parameters:
			feeds_required (bool) : A bool representing the feeds_required
		"""

		if feeds_required is not None and not isinstance(feeds_required, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: feeds_required EXPECTED TYPE: bool', None, None)
		
		self.__feeds_required = feeds_required
		self.__key_modified['feeds_required'] = 1

	def get_scoring_supported(self):
		"""
		The method to get the scoring_supported

		Returns:
			bool: A bool representing the scoring_supported
		"""

		return self.__scoring_supported

	def set_scoring_supported(self, scoring_supported):
		"""
		The method to set the value to scoring_supported

		Parameters:
			scoring_supported (bool) : A bool representing the scoring_supported
		"""

		if scoring_supported is not None and not isinstance(scoring_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: scoring_supported EXPECTED TYPE: bool', None, None)
		
		self.__scoring_supported = scoring_supported
		self.__key_modified['scoring_supported'] = 1

	def get_webform_supported(self):
		"""
		The method to get the webform_supported

		Returns:
			bool: A bool representing the webform_supported
		"""

		return self.__webform_supported

	def set_webform_supported(self, webform_supported):
		"""
		The method to set the value to webform_supported

		Parameters:
			webform_supported (bool) : A bool representing the webform_supported
		"""

		if webform_supported is not None and not isinstance(webform_supported, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: webform_supported EXPECTED TYPE: bool', None, None)
		
		self.__webform_supported = webform_supported
		self.__key_modified['webform_supported'] = 1

	def get_arguments(self):
		"""
		The method to get the arguments

		Returns:
			list: An instance of list
		"""

		return self.__arguments

	def set_arguments(self, arguments):
		"""
		The method to set the value to arguments

		Parameters:
			arguments (list) : An instance of list
		"""

		if arguments is not None and not isinstance(arguments, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: arguments EXPECTED TYPE: list', None, None)
		
		self.__arguments = arguments
		self.__key_modified['arguments'] = 1

	def get_module_name(self):
		"""
		The method to get the module_name

		Returns:
			string: A string representing the module_name
		"""

		return self.__module_name

	def set_module_name(self, module_name):
		"""
		The method to set the value to module_name

		Parameters:
			module_name (string) : A string representing the module_name
		"""

		if module_name is not None and not isinstance(module_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module_name EXPECTED TYPE: str', None, None)
		
		self.__module_name = module_name
		self.__key_modified['module_name'] = 1

	def get_business_card_field_limit(self):
		"""
		The method to get the business_card_field_limit

		Returns:
			int: An int representing the business_card_field_limit
		"""

		return self.__business_card_field_limit

	def set_business_card_field_limit(self, business_card_field_limit):
		"""
		The method to set the value to business_card_field_limit

		Parameters:
			business_card_field_limit (int) : An int representing the business_card_field_limit
		"""

		if business_card_field_limit is not None and not isinstance(business_card_field_limit, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: business_card_field_limit EXPECTED TYPE: int', None, None)
		
		self.__business_card_field_limit = business_card_field_limit
		self.__key_modified['business_card_field_limit'] = 1

	def get_custom_view(self):
		"""
		The method to get the custom_view

		Returns:
			CustomView: An instance of CustomView
		"""

		return self.__custom_view

	def set_custom_view(self, custom_view):
		"""
		The method to set the value to custom_view

		Parameters:
			custom_view (CustomView) : An instance of CustomView
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.customviews import CustomView
		except Exception:
			from ..custom_views import CustomView

		if custom_view is not None and not isinstance(custom_view, CustomView):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: custom_view EXPECTED TYPE: CustomView', None, None)
		
		self.__custom_view = custom_view
		self.__key_modified['custom_view'] = 1

	def get_parent_module(self):
		"""
		The method to get the parent_module

		Returns:
			Module: An instance of Module
		"""

		return self.__parent_module

	def set_parent_module(self, parent_module):
		"""
		The method to set the value to parent_module

		Parameters:
			parent_module (Module) : An instance of Module
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.modules.module import Module
		except Exception:
			from .module import Module

		if parent_module is not None and not isinstance(parent_module, Module):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: parent_module EXPECTED TYPE: Module', None, None)
		
		self.__parent_module = parent_module
		self.__key_modified['parent_module'] = 1

	def get_territory(self):
		"""
		The method to get the territory

		Returns:
			Territory: An instance of Territory
		"""

		return self.__territory

	def set_territory(self, territory):
		"""
		The method to set the value to territory

		Parameters:
			territory (Territory) : An instance of Territory
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.modules.territory import Territory
		except Exception:
			from .territory import Territory

		if territory is not None and not isinstance(territory, Territory):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: territory EXPECTED TYPE: Territory', None, None)
		
		self.__territory = territory
		self.__key_modified['territory'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Choice, Constants
except Exception:
	from ..exception import SDKException
	from ..util import Choice, Constants


class JobDetail(object):
	def __init__(self):
		"""Creates an instance of JobDetail"""

		self.__id = None
		self.__operation = None
		self.__state = None
		self.__query = None
		self.__created_by = None
		self.__created_time = None
		self.__result = None
		self.__file_type = None
		self.__key_modified = dict()

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_operation(self):
		"""
		The method to get the operation

		Returns:
			string: A string representing the operation
		"""

		return self.__operation

	def set_operation(self, operation):
		"""
		The method to set the value to operation

		Parameters:
			operation (string) : A string representing the operation
		"""

		if operation is not None and not isinstance(operation, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: operation EXPECTED TYPE: str', None, None)
		
		self.__operation = operation
		self.__key_modified['operation'] = 1

	def get_state(self):
		"""
		The method to get the state

		Returns:
			Choice: An instance of Choice
		"""

		return self.__state

	def set_state(self, state):
		"""
		The method to set the value to state

		Parameters:
			state (Choice) : An instance of Choice
		"""

		if state is not None and not isinstance(state, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: state EXPECTED TYPE: Choice', None, None)
		
		self.__state = state
		self.__key_modified['state'] = 1

	def get_query(self):
		"""
		The method to get the query

		Returns:
			Query: An instance of Query
		"""

		return self.__query

	def set_query(self, query):
		"""
		The method to set the value to query

		Parameters:
			query (Query) : An instance of Query
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.query import Query
		except Exception:
			from .query import Query

		if query is not None and not isinstance(query, Query):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: query EXPECTED TYPE: Query', None, None)
		
		self.__query = query
		self.__key_modified['query'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['created_by'] = 1

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['created_time'] = 1

	def get_result(self):
		"""
		The method to get the result

		Returns:
			Result: An instance of Result
		"""

		return self.__result

	def set_result(self, result):
		"""
		The method to set the value to result

		Parameters:
			result (Result) : An instance of Result
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.result import Result
		except Exception:
			from .result import Result

		if result is not None and not isinstance(result, Result):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: result EXPECTED TYPE: Result', None, None)
		
		self.__result = result
		self.__key_modified['result'] = 1

	def get_file_type(self):
		"""
		The method to get the file_type

		Returns:
			string: A string representing the file_type
		"""

		return self.__file_type

	def set_file_type(self, file_type):
		"""
		The method to set the value to file_type

		Parameters:
			file_type (string) : A string representing the file_type
		"""

		if file_type is not None and not isinstance(file_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_type EXPECTED TYPE: str', None, None)
		
		self.__file_type = file_type
		self.__key_modified['file_type'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import APIResponse, CommonAPIHandler, Constants
	from zcrmsdk.src.com.zoho.crm.api.param import Param
except Exception:
	from ..exception import SDKException
	from ..util import APIResponse, CommonAPIHandler, Constants
	from ..param import Param


class FieldAttachmentsOperations(object):
	def __init__(self, module_api_name, record_id, fields_attachment_id=None):
		"""
		Creates an instance of FieldAttachmentsOperations with the given parameters

		Parameters:
			module_api_name (string) : A string representing the module_api_name
			record_id (int) : An int representing the record_id
			fields_attachment_id (int) : An int representing the fields_attachment_id
		"""

		if not isinstance(module_api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module_api_name EXPECTED TYPE: str', None, None)
		
		if not isinstance(record_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: record_id EXPECTED TYPE: int', None, None)
		
		if fields_attachment_id is not None and not isinstance(fields_attachment_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: fields_attachment_id EXPECTED TYPE: int', None, None)
		
		self.__module_api_name = module_api_name
		self.__record_id = record_id
		self.__fields_attachment_id = fields_attachment_id


	def get_field_attachments(self):
		"""
		The method to get field attachments

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/actions/download_fields_attachment'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.add_param(Param('fields_attachment_id', 'com.zoho.crm.api.FieldAttachments.GetFieldAttachmentsParam'), self.__fields_attachment_id)
		try:
			from zcrmsdk.src.com.zoho.crm.api.field_attachments.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/x-download')


class GetFieldAttachmentsParam(object):
	pass



from .api_exception import APIException
from .response_handler import ResponseHandler
from .attachment import Attachment
from .email_templates_operations import EmailTemplatesOperations, GetEmailTemplatesParam, GetEmailTemplatebyIDParam
from .response_wrapper import ResponseWrapper
from .email_template import EmailTemplate



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Translation(object):
	def __init__(self):
		"""Creates an instance of Translation"""

		self.__public_views = None
		self.__other_users_views = None
		self.__shared_with_me = None
		self.__created_by_me = None
		self.__key_modified = dict()

	def get_public_views(self):
		"""
		The method to get the public_views

		Returns:
			string: A string representing the public_views
		"""

		return self.__public_views

	def set_public_views(self, public_views):
		"""
		The method to set the value to public_views

		Parameters:
			public_views (string) : A string representing the public_views
		"""

		if public_views is not None and not isinstance(public_views, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: public_views EXPECTED TYPE: str', None, None)
		
		self.__public_views = public_views
		self.__key_modified['public_views'] = 1

	def get_other_users_views(self):
		"""
		The method to get the other_users_views

		Returns:
			string: A string representing the other_users_views
		"""

		return self.__other_users_views

	def set_other_users_views(self, other_users_views):
		"""
		The method to set the value to other_users_views

		Parameters:
			other_users_views (string) : A string representing the other_users_views
		"""

		if other_users_views is not None and not isinstance(other_users_views, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: other_users_views EXPECTED TYPE: str', None, None)
		
		self.__other_users_views = other_users_views
		self.__key_modified['other_users_views'] = 1

	def get_shared_with_me(self):
		"""
		The method to get the shared_with_me

		Returns:
			string: A string representing the shared_with_me
		"""

		return self.__shared_with_me

	def set_shared_with_me(self, shared_with_me):
		"""
		The method to set the value to shared_with_me

		Parameters:
			shared_with_me (string) : A string representing the shared_with_me
		"""

		if shared_with_me is not None and not isinstance(shared_with_me, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: shared_with_me EXPECTED TYPE: str', None, None)
		
		self.__shared_with_me = shared_with_me
		self.__key_modified['shared_with_me'] = 1

	def get_created_by_me(self):
		"""
		The method to get the created_by_me

		Returns:
			string: A string representing the created_by_me
		"""

		return self.__created_by_me

	def set_created_by_me(self, created_by_me):
		"""
		The method to set the value to created_by_me

		Parameters:
			created_by_me (string) : A string representing the created_by_me
		"""

		if created_by_me is not None and not isinstance(created_by_me, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by_me EXPECTED TYPE: str', None, None)
		
		self.__created_by_me = created_by_me
		self.__key_modified['created_by_me'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
    import threading
    import logging
    import enum
    import json
    import time
    import requests
    from .token import Token
    from zcrmsdk.src.com.zoho.crm.api.initializer import Initializer
    from ...crm.api.util import APIHTTPConnector
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
    from ...crm.api.util.constants import Constants

except Exception as e:
    import threading
    import logging
    import enum
    import json
    import time
    import requests
    from .token import Token
    from zcrmsdk.src.com.zoho.crm.api.initializer import Initializer
    from ...crm.api.util import APIHTTPConnector
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
    from ...crm.api.util.constants import Constants


class OAuthToken(Token):
    """
    This class maintains the tokens and authenticates every request.
    """

    logger = logging.getLogger('SDKLogger')
    lock = threading.Lock()

    def __init__(self, client_id, client_secret, grant_token=None, refresh_token=None, redirect_url=None, id=None):

        """
        Creates an OAuthToken class instance with the specified parameters.

        Parameters:
            client_id (str) : A string containing the OAuth client id.
            client_secret (str) : A string containing the OAuth client secret.
            grant_token (str) : A string containing the GRANT token.
            refresh_token (str) : A string containing the REFRESH token.
            redirect_url (str) : A string containing the OAuth redirect URL. Default value is None
            id (str) : A string containing the Id. Default value is None
        """

        error = {}

        if not isinstance(client_id, str):
            error[Constants.FIELD] = Constants.CLIENT_ID
            error[Constants.EXPECTED_TYPE] = Constants.STRING
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.TOKEN_ERROR, details=error)

        if not isinstance(client_secret, str):
            error[Constants.FIELD] = Constants.CLIENT_SECRET
            error[Constants.EXPECTED_TYPE] = Constants.STRING
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.TOKEN_ERROR, details=error)

        if grant_token is None and refresh_token is None:
            error[Constants.FIELD] = Constants.TYPE
            error[Constants.EXPECTED_TYPE] = Constants.EXPECTED_TOKEN_TYPES
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.INPUT_ERROR, details=error)

        if grant_token is not None and not isinstance(grant_token, str):
            error[Constants.FIELD] = Constants.GRANT_TOKEN
            error[Constants.EXPECTED_TYPE] = Constants.STRING
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.TOKEN_ERROR, details=error)

        if refresh_token is not None and not isinstance(refresh_token, str):
            error[Constants.FIELD] = Constants.REFRESH_TOKEN
            error[Constants.EXPECTED_TYPE] = Constants.STRING
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.TOKEN_ERROR, details=error)

        if redirect_url is not None and not isinstance(redirect_url, str):
            error[Constants.FIELD] = Constants.REDIRECT_URI
            error[Constants.EXPECTED_TYPE] = Constants.STRING
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.TOKEN_ERROR, details=error)

        if id is not None and not isinstance(id, str):
            error[Constants.FIELD] = Constants.ID
            error[Constants.EXPECTED_TYPE] = Constants.STRING
            error[Constants.CLASS] = OAuthToken.__name__
            raise SDKException(code=Constants.TOKEN_ERROR, details=error)

        self.__client_id = client_id
        self.__client_secret = client_secret
        self.__redirect_url = redirect_url
        self.__grant_token = grant_token
        self.__refresh_token = refresh_token
        self.__id = id
        self.__access_token = None
        self.__expires_in = None
        self.__user_mail = None

    def get_client_id(self):
        """
        This is a getter method to get __client_id.

        Returns:
            string: A string representing __client_id
        """

        return self.__client_id

    def get_client_secret(self):
        """
        This is a getter method to get __client_secret.

        Returns:
            string: A string representing __client_secret
        """

        return self.__client_secret

    def get_redirect_url(self):
        """
        This is a getter method to get __redirect_url.

        Returns:
            string: A string representing __redirect_url
        """

        return self.__redirect_url

    def get_grant_token(self):
        """
        This is a getter method to get __grant_token.

        Returns:
            string: A string representing __grant_token
        """
        return self.__grant_token

    def get_refresh_token(self):
        """
        This is a getter method to get __refresh_token.

        Returns:
            string: A string representing __refresh_token
        """

        return self.__refresh_token

    def get_access_token(self):
        """
        This is a getter method to get __access_token.

        Returns:
            string: A string representing __access_token
        """

        return self.__access_token

    def get_id(self):
        """
        This is a getter method to get __id.

        Returns:
            string: A string representing __id
        """

        return self.__id

    def get_expires_in(self):
        """
        This is a getter method to get __expires_in.

        Returns:
            string: A string representing __expires_in
        """

        return self.__expires_in

    def get_user_mail(self):
        """
        This is a getter method to get __user_mail.

        Returns:
            string: A string representing __user_mail
        """

        return self.__user_mail

    def set_refresh_token(self, refresh_token):
        """
        This is a setter method to set __refresh_token.

        """
        self.__refresh_token = refresh_token

    def set_access_token(self, access_token):
        """
        This is a setter method to set __access_token.

        """

        self.__access_token = access_token

    def set_id(self, id):
        """
        This is a setter method to set __id.

        """

        self.__id = id

    def set_expires_in(self, expires_in):
        """
        This is a setter method to set __expires_in.

        """

        self.__expires_in = expires_in

    def set_user_mail(self, user_mail):
        """
        This is a setter method to set __user_mail.

        """

        self.__user_mail = user_mail


    def authenticate(self, url_connection):
        with OAuthToken.lock:
            initializer = Initializer.get_initializer()
            store = initializer.store
            user = initializer.user

            if self.__id is not None:
                oauth_token = initializer.store.get_token_by_id(self.__id, self)
            else:
                oauth_token = initializer.store.get_token(user, self)

            if oauth_token is None:
                token = self.generate_access_token(user, store).get_access_token() if (
                            self.__refresh_token is None) else self.refresh_access_token(user, store).get_access_token()

            elif int(oauth_token.get_expires_in()) - int(time.time() * 1000) < 5000:
                OAuthToken.logger.info(Constants.REFRESH_TOKEN_MESSAGE)
                token = oauth_token.refresh_access_token(user, store).get_access_token()

            else:
                token = oauth_token.get_access_token()

            url_connection.add_header(Constants.AUTHORIZATION, Constants.OAUTH_HEADER_PREFIX + token)

    def refresh_access_token(self, user, store):
        try:
            url = Initializer.get_initializer().environment.accounts_url

            body = {
                Constants.REFRESH_TOKEN: self.__refresh_token,
                Constants.CLIENT_ID: self.__client_id,
                Constants.REDIRECT_URI: self.__redirect_url if self.__redirect_url is not None else None,
                Constants.CLIENT_SECRET: self.__client_secret,
                Constants.GRANT_TYPE: Constants.REFRESH_TOKEN
            }

            response = requests.post(url, data=body, params=None, headers=None, allow_redirects=False).json()
            self.parse_response(response)
            self.generate_id()
            store.save_token(user, self)

        except SDKException as ex:
            raise ex

        except Exception as ex:
            raise SDKException(code=Constants.SAVE_TOKEN_ERROR, cause=ex)

        return self

    def generate_access_token(self, user, store):
        try:
            url = Initializer.get_initializer().environment.accounts_url

            body = {
                Constants.GRANT_TYPE: Constants.GRANT_TYPE_AUTH_CODE,
                Constants.CLIENT_ID: self.__client_id,
                Constants.CLIENT_SECRET: self.__client_secret,
                Constants.REDIRECT_URI: self.__redirect_url if self.__redirect_url is not None else None,
                Constants.CODE: self.__grant_token
            }
            headers = dict()
            headers[Constants.USER_AGENT_KEY] = Constants.USER_AGENT
            response = requests.post(url, data=body, params=None, headers=headers, allow_redirects=True).json()
            self.parse_response(response)
            self.generate_id()
            store.save_token(user, self)

        except SDKException as ex:
            raise ex

        except Exception as ex:
            raise SDKException(code=Constants.SAVE_TOKEN_ERROR, cause=ex)

        return self

    def parse_response(self, response):
        response_json = dict(response)

        if Constants.ACCESS_TOKEN not in response_json:
            raise SDKException(code=Constants.INVALID_TOKEN_ERROR, message=str(response_json.get(Constants.ERROR_KEY))if Constants.ERROR_KEY in response_json else Constants.NO_ACCESS_TOKEN_ERROR)

        self.__access_token = response_json.get(Constants.ACCESS_TOKEN)
        self.__expires_in = str(int(time.time() * 1000) + self.get_token_expires_in(response=response_json))

        if Constants.REFRESH_TOKEN in response_json:
            self.__refresh_token = response_json.get(Constants.REFRESH_TOKEN)

        return self

    @staticmethod
    def get_token_expires_in(response):
        return int(response[Constants.EXPIRES_IN]) if Constants.EXPIRES_IN_SEC in response else int(
            response[Constants.EXPIRES_IN]) * 1000

    def generate_id(self):
        id = ""
        email = str(Initializer.get_initializer().user.get_email())
        environment = str(Initializer.get_initializer().environment.name)
        id += "python_" + email[0:email.find(Constants.AT)] + Constants.UNDERSCORE
        id += environment + Constants.UNDERSCORE + self.__refresh_token[len(self.__refresh_token)-4:]
        self.__id = id
        return self.__id

    def remove(self):
        try:
            Initializer.get_initializer().store.delete_token(self)
            return True
        except Exception:
            return False



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
	from zcrmsdk.src.com.zoho.crm.api.record.record import Record
except Exception:
	from ..exception import SDKException
	from ..util import Constants
	from .record import Record


class PriceBook(Record):
	def __init__(self):
		"""Creates an instance of PriceBook"""
		super().__init__()


	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.get_key_value('name')

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.add_key_value('name', name)



from .api_exception import APIException
from .response_handler import ResponseHandler
from .file_body_wrapper import FileBodyWrapper
from .field_attachments_operations import FieldAttachmentsOperations, GetFieldAttachmentsParam
from .body_wrapper import BodyWrapper



try:
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
    from zcrmsdk.src.com.zoho.crm.api.util.constants import Constants
except Exception:
    from .exception import SDKException
    from .util import Constants


class RequestProxy(object):
    def __init__(self, host, port, user=None, password=""):

        """
        Creates a RequestProxy class instance with the specified parameters.

        Parameters:
            host(str): A String containing the hostname or address of the proxy server
            port(int): An Integer containing The port number of the proxy server
            user(str): A String containing the user name of the proxy server
            password(str) : A String containing the password of the proxy server. Default value is an empty string

        Raises:
            SDKException
        """

        if host is None:
            raise SDKException(Constants.USER_PROXY_ERROR, Constants.HOST_ERROR_MESSAGE)

        if port is None:
            raise SDKException(Constants.USER_PROXY_ERROR, Constants.PORT_ERROR_MESSAGE)

        self.__host = host
        self.__port = port
        self.__user = user
        self.__password = password

    def get_host(self):
        """
        This is a getter method to get __host.

        Returns:
            string: A string representing __host
        """

    def get_port(self):
        """
        This is a getter method to get __port.

        Returns:
            string: A string representing __port
        """

    def get_user(self):
        """
        This is a getter method to get __user.

        Returns:
            string: A string representing __user
        """

    def get_password(self):
        """
        This is a getter method to get __password.

        Returns:
            string: A string representing __password
        """


try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class ChartData(object):
	def __init__(self):
		"""Creates an instance of ChartData"""

		self.__nodes = None
		self.__connections = None
		self.__canvas_width = None
		self.__canvas_height = None
		self.__key_modified = dict()

	def get_nodes(self):
		"""
		The method to get the nodes

		Returns:
			list: An instance of list
		"""

		return self.__nodes

	def set_nodes(self, nodes):
		"""
		The method to set the value to nodes

		Parameters:
			nodes (list) : An instance of list
		"""

		if nodes is not None and not isinstance(nodes, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: nodes EXPECTED TYPE: list', None, None)
		
		self.__nodes = nodes
		self.__key_modified['nodes'] = 1

	def get_connections(self):
		"""
		The method to get the connections

		Returns:
			list: An instance of list
		"""

		return self.__connections

	def set_connections(self, connections):
		"""
		The method to set the value to connections

		Parameters:
			connections (list) : An instance of list
		"""

		if connections is not None and not isinstance(connections, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: connections EXPECTED TYPE: list', None, None)
		
		self.__connections = connections
		self.__key_modified['connections'] = 1

	def get_canvas_width(self):
		"""
		The method to get the canvas_width

		Returns:
			int: An int representing the canvas_width
		"""

		return self.__canvas_width

	def set_canvas_width(self, canvas_width):
		"""
		The method to set the value to canvas_width

		Parameters:
			canvas_width (int) : An int representing the canvas_width
		"""

		if canvas_width is not None and not isinstance(canvas_width, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: canvas_width EXPECTED TYPE: int', None, None)
		
		self.__canvas_width = canvas_width
		self.__key_modified['canvas_width'] = 1

	def get_canvas_height(self):
		"""
		The method to get the canvas_height

		Returns:
			int: An int representing the canvas_height
		"""

		return self.__canvas_height

	def set_canvas_height(self, canvas_height):
		"""
		The method to set the value to canvas_height

		Parameters:
			canvas_height (int) : An int representing the canvas_height
		"""

		if canvas_height is not None and not isinstance(canvas_height, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: canvas_height EXPECTED TYPE: int', None, None)
		
		self.__canvas_height = canvas_height
		self.__key_modified['canvas_height'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Preference(object):
	def __init__(self):
		"""Creates an instance of Preference"""

		self.__auto_populate_tax = None
		self.__modify_tax_rates = None
		self.__key_modified = dict()

	def get_auto_populate_tax(self):
		"""
		The method to get the auto_populate_tax

		Returns:
			bool: A bool representing the auto_populate_tax
		"""

		return self.__auto_populate_tax

	def set_auto_populate_tax(self, auto_populate_tax):
		"""
		The method to set the value to auto_populate_tax

		Parameters:
			auto_populate_tax (bool) : A bool representing the auto_populate_tax
		"""

		if auto_populate_tax is not None and not isinstance(auto_populate_tax, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: auto_populate_tax EXPECTED TYPE: bool', None, None)
		
		self.__auto_populate_tax = auto_populate_tax
		self.__key_modified['auto_populate_tax'] = 1

	def get_modify_tax_rates(self):
		"""
		The method to get the modify_tax_rates

		Returns:
			bool: A bool representing the modify_tax_rates
		"""

		return self.__modify_tax_rates

	def set_modify_tax_rates(self, modify_tax_rates):
		"""
		The method to set the value to modify_tax_rates

		Parameters:
			modify_tax_rates (bool) : A bool representing the modify_tax_rates
		"""

		if modify_tax_rates is not None and not isinstance(modify_tax_rates, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modify_tax_rates EXPECTED TYPE: bool', None, None)
		
		self.__modify_tax_rates = modify_tax_rates
		self.__key_modified['modify_tax_rates'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



from .action_wrapper import ActionWrapper
from .base_currency_wrapper import BaseCurrencyWrapper
from .currencies_operations import CurrenciesOperations
from .format import Format
from .body_wrapper import BodyWrapper
from .base_currency_action_handler import BaseCurrencyActionHandler
from .api_exception import APIException
from .base_currency_action_wrapper import BaseCurrencyActionWrapper
from .action_response import ActionResponse
from .response_handler import ResponseHandler
from .success_response import SuccessResponse
from .response_wrapper import ResponseWrapper
from .action_handler import ActionHandler
from .currency import Currency



try:
    from abc import ABC, abstractmethod

except Exception as e:
    from abc import ABC, abstractmethod


class TokenStore(ABC):

    """
    This class is to store user token details.
    """

    @abstractmethod
    def get_token(self, user, token):

        """
        The method to get user token details.

        Parameters:
            user (UserSignature) : A UserSignature class instance.
            token (Token) : A Token class instance.

        Returns:
            Token : A Token class instance representing the user token details.
        """

        pass

    @abstractmethod
    def save_token(self, user, token):

        """
        The method to store user token details.

        Parameters:
            user (UserSignature) : A UserSignature class instance.
            token (Token) : A Token class instance.

        Returns:
            Token : A Token class instance representing the user token details.
        """

        pass

    @abstractmethod
    def delete_token(self, token):

        """
        The method to delete user token details.

        Parameters:
            token (Token) : A Token class instance.

        Returns:
            Token : A Token class instance representing the user token details.
        """

        pass

    @abstractmethod
    def get_tokens(self):

        """
        The method to retrieve all the stored tokens.

        Returns:
            list : A List of Token instances
        """

        pass

    @abstractmethod
    def delete_tokens(self):

        """
        The method to delete all the stored tokens.
        """

    pass
    @abstractmethod
    def get_token_by_id(self, id, token):

        """
        The method to get id token details.

        Parameters:
            id (String) : A String id.
            token (Token) : A Token class instance.

        Returns:
            Token : A Token class instance representing the id token details.
        """

    pass
